'From Squeak3.1alpha [latest update: #''Squeak3.1alpha'' of 30 March 2001 update 3910] on 24 April 2001 at 9:49:46 am'!
Magnitude subclass: #Date
	instanceVariableNames: 'julianDayNumber '
	classVariableNames: 'DaysInMonth FirstDayOfMonth MonthNames SecondsInDay WeekDayNames '
	poolDictionaries: ''
	category: 'Kernel-Chronology'!
Timespan subclass: #TDate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Chronology'!
DateAndTime subclass: #TTime
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Chronology'!
Magnitude subclass: #Time
	instanceVariableNames: 'seconds '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Chronology'!

!Celeste class methodsFor: 'sending' stamp: 'BP 3/30/2001 14:07'!
eudoraSeparator
	"Return a Eudora-style message separator string."

	| s today dateString |
	s _ WriteStream on: (String new: 50).
	today _ Date today.
	dateString _ today printFormat: #(2 1 3 32 2 1).
	dateString _ dateString copyFrom: 1 to: dateString size - 4.
	s nextPutAll: 'From ???@??? '.
	s nextPutAll: (today dayOfWeekName copyFrom: 1 to: 3); space.
	s nextPutAll: dateString.
	Time now print24: true on: s.
	s space.
	s print: today year; cr.
	^s contents
! !


!MailMessage methodsFor: 'parsing' stamp: 'BP 3/23/2001 12:47'!
readDateFrom: aStream
	"Parse a date from the given stream and answer nil if the date can't be parsed. The date may be in any of the following forms:
		<day> <monthName> <year>		(5 April 1982; 5-APR-82)
		<monthName> <day> <year>		(April 5, 1982)
		<monthNumber> <day> <year>		(4/5/82)
	In addition, the date may be preceded by the day of the week and an optional comma, such as:
		Tue, November 14, 1989"

	| day month year |
	self skipWeekdayName: aStream.
	aStream peek isDigit ifTrue: [day _ Integer readFrom: aStream].
	[aStream peek isAlphaNumeric] whileFalse: [aStream skip: 1].
	aStream peek isLetter
		ifTrue:		"month name or weekday name"
			[month _ WriteStream on: (String new: 10).
			 [aStream peek isLetter] whileTrue: [month nextPut: aStream next].
			 month _ month contents.
			 day isNil ifTrue:		"name/number..."
				[[aStream peek isAlphaNumeric] whileFalse: [aStream skip: 1].
				 (aStream peek isDigit) ifFalse: [^nil].
				 day _ Integer readFrom: aStream]]
		ifFalse:		"number/number..."
			[month _ Month nameOfMonth: day.
			 day _ Integer readFrom: aStream].
	[aStream peek isAlphaNumeric] whileFalse: [aStream skip: 1].
	(aStream peek isDigit) ifFalse: [^nil].
	year _ Integer readFrom: aStream.
	^Date newDay: day month: month year: year! !


!MailMessage class methodsFor: 'utilities' stamp: 'BP 3/30/2001 14:08'!
dateStampNow
	"Return the current date and time formatted as a email Date: line"
	"The result conforms to RFC822 with a long year, e.g.  'Thu, 18 Feb 1999 20:38:51'"

	^	(Date today dayOfWeekName copyFrom: 1 to: 3), ', ',
		(Date today printFormat: #(1 2 3 $  2 1 1)), ' ',
		Time now print24! !


!PDA methodsFor: 'initialization' stamp: 'BP 3/30/2001 14:06'!
labelString

	| today |
	today _ Date today.
	^ String streamContents:
		[:s | s nextPutAll: today dayOfWeekName; space.
		Time now print24: false showSeconds: false on: s.
		s nextPutAll: '  --  '.
		s nextPutAll: today monthName; space; print: today dayOfMonth;
			nextPutAll: ', '; print: today year]! !

!PDA methodsFor: 'initialization' stamp: 'BP 3/30/2001 14:07'!
openMonthView
	| row month col paneExtent window paneColor nRows |
	month _ date notNil
		ifTrue: [date month]
		ifFalse: ["But... it's here somewhere..."
				((self dependents detect: [:m | m isKindOf: PDAMorph])
					findA: MonthMorph) month].
	window _ SystemWindow labelled: month printString.
	paneColor _ Color transparent.
	window color: (Color r: 0.968 g: 1.0 b: 0.355).
	nRows _ 0.  month eachWeekDo: [:w | nRows _ nRows + 1].
	paneExtent _ ((1.0/7) @ (1.0/nRows)).
	row _ 0.
	month eachWeekDo:
		[:week | col _ 0.
		week do:
			[:day | day month = month ifTrue:
				[window addMorph: ((PluggableListMorph on: self list: nil
						selected: nil changeSelected: nil menu: nil keystroke: nil)
							list: {(day dayOfMonth printString , '  ' , day dayOfWeekName) asText allBold}
								, (self scheduleListForDay: day))
					frame: (paneExtent * (col@row) extent: paneExtent)].
			col _ col + 1].
		row _ row + 1].

	window firstSubmorph color: paneColor.
	window updatePaneColors.
	window openInWorld! !

!PDA methodsFor: 'example' stamp: 'BP 3/30/2001 16:09'!
sampleScheduleList

	^ {
	PDAEvent new key: 'home'; date: Date today; description: 'wake up'; time: (Time hour: 6).
	PDAEvent new key: 'home'; date: Date today; description: 'go for a run'; time: (Time hour: 7).
	PDAEvent new key: 'home'; date: Date today; description: 'take a shower'; time: (Time hour: 8).
	PDAEvent new key: 'home'; date: (Date today addDays: 2); description: 'dinner out'; time: (Time hour: 18).
	PDAEvent new key: 'work'; date: (Date today addDays: 1); description: 'conf call'; time: (Time hour: 10).
	PDAEvent new key: 'work'; date: (Date today addDays: 2); description: 'Leave for Conference'; time: (Time hour: 8).
	PDAEvent new key: 'work'; date: Date today; description: 'call Boss'; time: (Time hour: 15).
	PDAEvent new key: 'work'; date: Date today; description: 'Call about 401k'; time: (Time hour: 10).
	}! !


!PDAMorph methodsFor: 'as yet unclassified' stamp: 'BP 3/30/2001 16:19'!
stepTime

	^ (60 - Time now second + 1) * 1000
! !


!PDARecurringEvent methodsFor: 'as yet unclassified' stamp: 'BP 3/30/2001 14:09'!
matchesDate: aDate

	(firstDate == nil or: [firstDate > aDate]) ifTrue: [^ false].
	(lastDate ~~ nil and: [lastDate < aDate]) ifTrue: [^ false].
	recurrence == #eachDay ifTrue:
		[^ true].
	recurrence == #dayOfWeek ifTrue:
		[^ aDate dayOfWeekName = firstDate dayOfWeekName].
	recurrence == #dayOfMonth ifTrue:
		[^ aDate dayOfMonth = firstDate dayOfMonth].
	recurrence == #dateOfYear ifTrue:
		[^ aDate monthIndex = firstDate monthIndex
			and: [aDate dayOfMonth = firstDate dayOfMonth]].
	recurrence == #nthWeekdayOfMonth ifTrue:
		[^ aDate dayOfWeekName = firstDate dayOfWeekName
			and: [(aDate dayOfMonth - 1 // 7) = (firstDate dayOfMonth - 1 // 7)]].
	recurrence == #nthWeekdayOfMonthEachYear ifTrue:
		[^ aDate monthIndex = firstDate monthIndex
			and: [aDate dayOfWeekName = firstDate dayOfWeekName
			and: [(aDate dayOfMonth - 1 // 7) = (firstDate dayOfMonth - 1 // 7)]]].
! !


!ServerDirectory methodsFor: 'accessing' stamp: 'BP 3/23/2001 12:44'!
secondsForDay: dayToken month: monthToken yearOrTime: ytToken 
thisMonth: thisMonth thisYear: thisYear

	| ftpDay ftpMonth pickAYear jDateToday trialJulianDate |

	ftpDay _ dayToken asNumber.
	ftpMonth _ Month indexOfMonth: monthToken.
	(ytToken includes: $:) ifFalse: [
		^(Date newDay: ftpDay month: ftpMonth year: ytToken asNumber) asSeconds
	].
	jDateToday _ Date today dayOfYear.
	trialJulianDate _ (Date newDay: ftpDay month: ftpMonth year: thisYear) dayOfYear.
	
	"Date has no year if within six months (do we need to check the day, too?)"

	"Well it appear to be pickier than that... it isn't just 6 months or 6 months and the day of the month, put perhaps the julian date AND the time as well. I don't know what the precise standard is, but this seems to produce better results"

	pickAYear _ (jDateToday - trialJulianDate) > 182 ifTrue: [
		thisYear + 1	"his clock could be ahead of ours??"
	] ifFalse: [
		pickAYear _ (trialJulianDate - jDateToday) > 182 ifTrue: [
			thisYear - 1
		] ifFalse: [
			thisYear
		].
	].
	^(Date newDay: ftpDay month: ftpMonth year: pickAYear) asSeconds +
		(Time readFrom: (ReadStream on: ytToken)) asSeconds

! !


!TDate methodsFor: 'private' stamp: 'BP 3/26/2001 17:04'!
gregorian

	^start gregorian! !

!TDate methodsFor: 'printing' stamp: 'BP 3/23/2001 12:27'!
mmddyyyy
	"Answer the receiver rendered in standard fmt mm/dd/yyyy.  Good for avoiding year 2000 bugs.  Note that the name here is slightly misleading -- the month and day numbers don't show leading zeros, so that for example feb 1 1996 is 2/1/96"

	"Date today mmddyyyy"

	^ self printFormat: #(2 1 3 $/ 1 1)! !

!TDate methodsFor: 'printing' stamp: 'BP 3/23/2001 12:27'!
printFormat: formatArray 
	"Answer a String describing the receiver using the argument formatArray."

	| aStream |
	aStream _ WriteStream on: (String new: 16).
	self printOn: aStream format: formatArray.
	^aStream contents! !

!TDate methodsFor: 'printing' stamp: 'BP 3/23/2001 12:27'!
printOn: aStream

	self printOn: aStream format: #(1 2 3 $  3 1 )! !

!TDate methodsFor: 'printing' stamp: 'BP 3/26/2001 17:05'!
printOn: aStream format: formatArray 
	"Print a description of the receiver on aStream using the format denoted the argument, formatArray:
		#(item item item sep monthfmt yearfmt twoDigits)
		items:  1=day  2=month  3=year  will appear in the order given,
		separated by sep which is eaither an ascii code or character.
		monthFmt:  1=09  2=Sep  3=September
		yearFmt:  1=1996  2=96
		digits:  (missing or)1=9  2=09.
	See the examples in printOn: and mmddyy"

	| gregorian twoDigits element monthFormat  |
	gregorian _ self gregorian.
	twoDigits _ formatArray size > 6 and: [ (formatArray at: 7) > 1 ].

	1 to: 3 do: 
	[ :i | 
		element _ formatArray at: i.
		element = 1 ifTrue:
		[ 
			twoDigits 
				ifTrue: [ aStream nextPutAll: (gregorian first asString padded: #left to: 2 with: $0) ]
				ifFalse: [ gregorian first printOn: aStream ]
		].
		element = 2 ifTrue: 
		[
			monthFormat _ formatArray at: 5.
			monthFormat = 1 ifTrue:
			[
				twoDigits 
					ifTrue: [ aStream nextPutAll: (gregorian middle asString padded: #left to: 2 with: $0) ]
					ifFalse: [ gregorian middle printOn: aStream ].
			].
			monthFormat = 2 ifTrue:
			[
				aStream nextPutAll: ((Month nameOfMonth: gregorian middle) copyFrom: 1 to: 3)
			].
			monthFormat = 3 ifTrue:
			[
				aStream nextPutAll: (Month nameOfMonth: gregorian middle)
			].
		].
		element = 3 ifTrue: 
		[
			(formatArray at: 6) = 1
				ifTrue: [ gregorian last printOn: aStream ]
				ifFalse: [ aStream nextPutAll: ((gregorian last \\ 100) asString padded: #left to: 2 with: $0) ].
		].
		i < 3 ifTrue: 
		[
			(formatArray at: 4) ~= 0 
				ifTrue: [ aStream nextPut: (formatArray at: 4) asCharacter ] 
		].
	].! !

!TDate methodsFor: 'printing' stamp: 'BP 3/23/2001 12:27'!
storeOn: aStream

	aStream print: self printString; nextPutAll: ' asDate'! !

!TDate methodsFor: 'smalltalk-80' stamp: 'BP 3/26/2001 16:34'!
addDays: dayCount 
	"obsolete"

	^self + dayCount! !

!TDate methodsFor: 'smalltalk-80' stamp: 'BP 3/26/2001 16:43'!
subtractDate: aDate 
	"obsolete"

	^(self start - aDate asDateAndTime) days! !

!TDate methodsFor: 'smalltalk-80' stamp: 'BP 3/26/2001 16:35'!
subtractDays: dayCount 
	"obsolete"

	^self - dayCount! !

!TDate methodsFor: 'squeak protocol' stamp: 'BP 3/23/2001 12:29'!
asDate

	^self! !

!TDate methodsFor: 'squeak protocol' stamp: 'BP 3/30/2001 15:00'!
asSeconds
	"Answer the seconds between a time on 1 January 1901 and the same 
	time in the receiver's day."

	^start asSeconds! !

!TDate methodsFor: 'squeak protocol' stamp: 'BP 3/26/2001 17:01'!
daysLeftInYear

	^start daysLeftInYear! !

!TDate methodsFor: 'squeak protocol' stamp: 'BP 3/27/2001 15:19'!
julianDayNumber

	^start julianDayNumber

! !

!TDate methodsFor: 'ansi protocol' stamp: 'BP 3/26/2001 17:08'!
+ anInteger
	"operand is the number of days"

	^self class starting: (start + (Duration day * anInteger))! !

!TDate methodsFor: 'ansi protocol' stamp: 'BP 3/26/2001 17:09'!
- anInteger
	"operand is the number of days"

	^self + (anInteger negated)! !

!TDate methodsFor: 'ansi protocol' stamp: 'BP 3/26/2001 16:46'!
dayOfMonth
	"Answer which day of the month is represented by the receiver."

	^start dayOfMonth! !

!TDate methodsFor: 'ansi protocol' stamp: 'BP 3/26/2001 16:38'!
dayOfWeek

	^start dayOfWeek	! !

!TDate methodsFor: 'ansi protocol' stamp: 'BP 3/30/2001 14:05'!
dayOfWeekName

	^start dayOfWeekName! !

!TDate methodsFor: 'ansi protocol' stamp: 'BP 3/26/2001 16:47'!
dayOfYear

	^start dayOfYear! !


!TDate class methodsFor: 'smalltalk-80' stamp: 'BP 3/23/2001 13:01'!
fromDays: dayCount

	^self julianDayNumber: dayCount +  2415386 "Julian Day Number of 1 Jan 1901" 
! !

!TDate class methodsFor: 'smalltalk-80' stamp: 'BP 3/23/2001 13:04'!
fromSeconds: seconds
	"Answer an instance of me which is 'seconds' seconds after January 1, 1901."

	^self fromDays: ((Duration seconds: seconds) days)! !

!TDate class methodsFor: 'smalltalk-80' stamp: 'BP 3/23/2001 12:58'!
newDay: day month: month year: year 

	^self year: year month: month day: day! !

!TDate class methodsFor: 'smalltalk-80' stamp: 'BP 3/23/2001 13:00'!
newDay: dayCount year: referenceYear 

	^self year: referenceYear day: dayCount! !

!TDate class methodsFor: 'squeak protocol' stamp: 'BP 3/23/2001 12:27'!
dateAndTimeNow
	"Answer an Array whose first element is Date today and second element 
	is Time now."

	^Time dateAndTimeNow! !

!TDate class methodsFor: 'squeak protocol' stamp: 'BP 3/23/2001 12:27'!
fromString: aString
	"Answer an instance of created from a string with format DD.MM.YYYY."

	^self readFrom: (ReadStream on: aString).
! !

!TDate class methodsFor: 'squeak protocol' stamp: 'BP 3/23/2001 12:51'!
julianDayNumber: aJulianDayNumber

	^self starting: (DateAndTime julianDayNumber: aJulianDayNumber)! !

!TDate class methodsFor: 'squeak protocol' stamp: 'BP 3/23/2001 13:02'!
readFrom: aStream
	"Read a Date from the stream in any of the forms:
		<day> <monthName> <year>		(5 April 1982; 5-APR-82)
		<monthName> <day> <year>		(April 5, 1982)
		<monthNumber> <day> <year>	(4/5/82)"

	| day month |
	aStream peek isDigit ifTrue: [day _ Integer readFrom: aStream].
	[aStream peek isAlphaNumeric] whileFalse: [aStream skip: 1].
	aStream peek isLetter
		ifTrue:		"number/name... or name..."
			[month _ WriteStream on: (String new: 10).
			[aStream peek isLetter] whileTrue: [month nextPut: aStream next].
			month _ month contents.
			day isNil ifTrue:		"name/number..."
				[[aStream peek isAlphaNumeric] whileFalse: [aStream skip: 1].
				day _ Integer readFrom: aStream]]
		ifFalse:		"number/number..."
			[month _ Date nameOfMonth: day.
			day _ Integer readFrom: aStream].
	[aStream peek isAlphaNumeric] whileFalse: [aStream skip: 1].

	^self year: (Integer readFrom: aStream) month: month day: day.

	"Date readFrom: (ReadStream on: '5APR82')"
! !

!TDate class methodsFor: 'squeak protocol' stamp: 'BP 3/23/2001 12:36'!
starting: aDateAndTime

	^super starting: (aDateAndTime midnight) duration: Duration day
! !

!TDate class methodsFor: 'squeak protocol' stamp: 'BP 3/23/2001 12:37'!
today

	^self current! !

!TDate class methodsFor: 'squeak protocol' stamp: 'BP 3/23/2001 12:55'!
year: year day: dayOfYear

	^self starting: (DateAndTime year: year day: dayOfYear)
! !

!TDate class methodsFor: 'squeak protocol' stamp: 'BP 3/23/2001 12:56'!
year: year month: month day: day

	^self starting: (DateAndTime year: year month: month day: day)
! !


!TTime methodsFor: 'smalltalk-80' stamp: 'BP 3/30/2001 15:25'!
addTime: timeAmount
	"Answer a Time that is timeInterval after the receiver. timeInterval is an 
	instance of Date or Time."

	^Time fromSeconds: self asSeconds + timeAmount asSeconds! !

!TTime methodsFor: 'smalltalk-80' stamp: 'BP 3/30/2001 15:25'!
subtractTime: timeAmount 
	"Answer a Time that is timeInterval before the receiver. timeInterval is 
	an instance of Date or Time."

	^Time fromSeconds: self asSeconds - timeAmount asSeconds! !

!TTime methodsFor: 'printing' stamp: 'BP 3/30/2001 15:25'!
hhmm24
	"Return a string of the form 1123 (for 11:23 am), 2154 (for 9:54 pm), of exactly 4 digits"

	^(String streamContents: 
		[ :aStream | self print24: true showSeconds: false on: aStream ])
			copyWithout: $:! !

!TTime methodsFor: 'printing' stamp: 'BP 3/30/2001 16:19'!
intervalString
	"Treat the time as a difference.  Give it in hours and minutes with two digits of accuracy."

	| hh mm ss |
	hh _ self hour = 0 ifTrue: [''] ifFalse: [' ', self hour printString, ' hours'].
	mm _ self minute = 0 ifTrue: [''] ifFalse: [' ', self minute printString, ' minutes'].
	ss _ self second = 0 ifTrue: [''] ifFalse: [' ', self second printString, ' seconds'].
	hh size > 0 ifTrue: [ss _ ''].
	^ hh, mm, ss! !

!TTime methodsFor: 'printing' stamp: 'BP 3/30/2001 15:25'!
print24
	"Return as 8-digit string 'hh:mm:ss', with leading zeros if needed"

	^String streamContents:
		[ :aStream | self print24: true on: aStream ]

! !

!TTime methodsFor: 'printing' stamp: 'BP 3/30/2001 15:25'!
print24: hr24 on: aStream 
	"Format is 'hh:mm:ss' or 'h:mm:ss am' "

	self print24: hr24 showSeconds: true on: aStream 
! !

!TTime methodsFor: 'printing' stamp: 'BP 3/30/2001 16:11'!
print24: hr24 showSeconds: showSeconds on: aStream 
	"Format is 'hh:mm:ss' or 'h:mm:ss am'  or, if showSeconds is false, 'hh:mm' or 'h:mm am'"

	| h m s |
	h _ self hour. m _ self minute. s _ self second.
	hr24
	ifTrue:
	[
		h < 10 ifTrue: [ aStream nextPutAll: '0' ].
		h printOn: aStream
	]
	ifFalse:
	[
		h > 12
		ifTrue: [ h - 12 printOn: aStream]
		ifFalse: 
		[
			h < 1
				ifTrue: [ 12 printOn: aStream ]
				ifFalse: [ h printOn: aStream ]
		]
	].

	aStream nextPutAll: (m < 10 ifTrue: [':0'] ifFalse: [':']).
	m printOn: aStream.

	showSeconds ifTrue:
	[
		aStream nextPutAll: (s < 10 ifTrue: [':0'] ifFalse: [':']).
		s printOn: aStream
	].

	hr24 ifFalse:
	[
		aStream nextPutAll: (h < 12 ifTrue: [' am'] ifFalse: [' pm'])
	].
! !

!TTime methodsFor: 'printing' stamp: 'BP 3/30/2001 15:25'!
printMinutes
	"Return as string 'hh:mm pm'  "

	^String streamContents:
		[ :aStream | self print24: false showSeconds: false on: aStream ]

! !

!TTime methodsFor: 'printing' stamp: 'BP 3/30/2001 16:18'!
printOn: aStream 

	self second = 0
		ifTrue: [self print24: false showSeconds: false on: aStream]
		ifFalse: [self print24: false on: aStream]! !

!TTime methodsFor: 'printing' stamp: 'BP 3/30/2001 15:25'!
storeOn: aStream

	aStream print: self printString; nextPutAll: ' asTime'! !

!TTime methodsFor: 'ansi protocol' stamp: 'BP 3/30/2001 15:46'!
< aTime 
	"Answer whether aTime is earlier than the receiver."

	^seconds < aTime asSeconds! !

!TTime methodsFor: 'ansi protocol' stamp: 'BP 3/30/2001 15:47'!
= aTime 
	"Answer whether aTime represents the same second as the receiver."

	self species = aTime species
		ifTrue: [ ^seconds = aTime asSeconds ]
		ifFalse: [ ^false ].! !

!TTime methodsFor: 'ansi protocol' stamp: 'BP 3/30/2001 15:25'!
hash
	"Hash must be redefined since = was redefined."

	^seconds hash! !

!TTime methodsFor: 'squeak protocol' stamp: 'BP 3/30/2001 15:25'!
asSeconds
	"Answer the number of seconds since midnight of the receiver."

	^seconds! !


!TTime class methodsFor: 'smalltalk-80' stamp: 'BP 3/30/2001 16:43'!
fromSeconds: secondCount 
	"Answer an instance of me that is secondCount number of seconds since midnight."

	^self seconds: secondCount
! !

!TTime class methodsFor: 'smalltalk-80' stamp: 'BP 3/30/2001 16:43'!
readFrom: aStream
	"Read a Time from the stream in the form:
		<hour>:<minute>:<second> <am/pm>

	<minute>, <second> or <am/pm> may be omitted.  e.g. 1:59:30 pm; 8AM; 15:30"

	| hour minute second ampm |
	hour _ Integer readFrom: aStream.
	minute _ 0.
	second _ 0.
	(aStream peekFor: $:) ifTrue:
	[
		minute _ Integer readFrom: aStream.
		(aStream peekFor: $:) ifTrue: [ second _ Integer readFrom: aStream ]
	].
	aStream skipSeparators.
	(aStream atEnd not and: [aStream peek isLetter])
	ifTrue:
	[
		ampm _ aStream next asLowercase.
		(ampm = $p and: [hour < 12]) ifTrue: [hour _ hour + 12].
		(ampm = $a and: [hour = 12]) ifTrue: [hour _ 0].
		(aStream peekFor: $m) ifFalse: [aStream peekFor: $M]
	].
	^self seconds: 60*(60*hour+minute)+second

	"Time readFrom: (ReadStream on: '2:23:09 pm')"
! !

!TTime class methodsFor: 'smalltalk-80' stamp: 'BP 3/30/2001 15:25'!
totalSeconds
	"Answer the total seconds from January 1, 1901."

	^ self primSecondsClock! !

!TTime class methodsFor: 'general inquiries' stamp: 'BP 3/30/2001 15:25'!
condenseBunches: aCollectionOfSeconds
	| secArray pause now out prev bunchEnd ago |
	"Identify the major intervals in a bunch of numbers.  Each number is a seconds since 1901 that represents a date and time.  We want the last event in a bunch.  Return array of seconds for:
	Every event in the last half hour.
	Every bunch separated by 30 min in the last 24 hours.
	Every bunch separated by two hours before that."
"Time condenseBunches: 
	(#(20 400 401  20000 20200 20300   40000 45000  200000 201000 202000) collect: [:tt | 
		self totalSeconds - tt]) 
"
	secArray _ aCollectionOfSeconds asSortedCollection.
	pause _ 1.
	now _ self totalSeconds.
	out _ OrderedCollection new.
	prev _ 0.
	bunchEnd _ nil.
	secArray reverseDo: [:secs | "descending"
		ago _ now - secs.
		ago > (60*30) ifTrue: [pause _ "60*30" 1800].
		ago > (60*60*24) ifTrue: [pause _ "60*120" 7200].
		ago - prev >= pause ifTrue: [out add: bunchEnd.  bunchEnd _ secs].
		prev _ ago].
	out add: bunchEnd.
	out removeFirst.
	^ out! !

!TTime class methodsFor: 'general inquiries' stamp: 'BP 3/30/2001 15:25'!
dateAndTimeFromSeconds: secondCount

	^ Array
		with: (Date fromSeconds: secondCount)
		with: (Time fromSeconds: secondCount \\ 86400)
! !

!TTime class methodsFor: 'general inquiries' stamp: 'BP 3/30/2001 15:25'!
dateAndTimeNow
	"Answer a two-element Array of (Date today, Time now)."

	| secondCount d t |
	secondCount _ self primSecondsClock.
	d _ Date fromSeconds: secondCount.
	t _ Time fromSeconds: secondCount \\ 86400.
	^ Array with: d with: t! !

!TTime class methodsFor: 'general inquiries' stamp: 'BP 3/30/2001 15:25'!
humanWordsForSecondsAgo: secs
	| date today |
	"Return natural language for this date and time in the past."

	secs <= 1 ifTrue: [^ 'a second ago'].
	secs < 45 ifTrue: [^ secs printString, ' seconds ago'].
	secs < 90 ifTrue: [^ 'a minute ago'].
	secs < "45*60" 2700 ifTrue: [^ (secs//60) printString, ' minutes ago'].
	secs < "90*60" 5400 ifTrue: [^ 'an hour ago'].
	secs < "18*60*60" 64800 ifTrue: [^ (secs//3600) printString, ' hours ago'].
	date _ Date fromSeconds: self totalSeconds - secs.		"now work with dates"
	today _ Date today.
	date > (today subtractDays: 2) ifTrue: [^ 'yesterday'].
	date > (today subtractDays: 8) ifTrue: [^ 'last ', date dayOfWeekName].
	date > (today subtractDays: 13) ifTrue: [^ 'a week ago'].
	date > (today subtractDays: 28) ifTrue: [
		^ ((today subtractDate: date)//7) printString, ' weeks ago'].
	date > (today subtractDays: 45) ifTrue: [^ 'a month ago'].
	date > (today subtractDays: 300) ifTrue: [^ 'last ', date monthName].
	^ date monthName, ', ', date year printString

"Example
#(0.5 30 62 130 4000 10000 60000 90000 345600 864000 1728000 3456000 17280000 34560000 345600000) collect: [:ss | Time humanWordsForSecondsAgo: ss].
"! !

!TTime class methodsFor: 'general inquiries' stamp: 'BP 3/30/2001 15:25'!
millisecondClockValue
	"Answer the number of milliseconds since the millisecond clock was last 
	reset or rolled over."

	^ self primMillisecondClock! !

!TTime class methodsFor: 'general inquiries' stamp: 'BP 3/30/2001 15:25'!
milliseconds: currentTime since: lastTime
	| delta |
	"Answer the elapsed time since last recorded in milliseconds.
	Compensate for rollover."

	delta _ currentTime - lastTime.
	^delta < 0
		ifTrue: [SmallInteger maxVal + delta]
		ifFalse: [delta]
! !

!TTime class methodsFor: 'general inquiries' stamp: 'BP 3/30/2001 15:25'!
millisecondsSince: lastTime
	"Answer the elapsed time since last recorded in milliseconds.
	Compensate for rollover."

	^self milliseconds: self millisecondClockValue since: lastTime! !

!TTime class methodsFor: 'general inquiries' stamp: 'BP 3/30/2001 15:25'!
millisecondsToRun: timedBlock 
	"Answer the number of milliseconds timedBlock takes to return its value."

	| initialMilliseconds |
	initialMilliseconds _ self millisecondClockValue.
	timedBlock value.
	^self millisecondClockValue - initialMilliseconds! !

!TTime class methodsFor: 'general inquiries' stamp: 'BP 3/30/2001 15:25'!
namesForTimes: arrayOfSeconds
	| simpleEnglish prev final prevPair myPair |
	"Return English descriptions of the times in the array.  They are each seconds since 1901.  If two names are the same, append the date and time to distinguish them."

	simpleEnglish _ arrayOfSeconds collect: [:secsAgo |
		self humanWordsForSecondsAgo: self totalSeconds - secsAgo].
	prev _ ''.
	final _ simpleEnglish copy.
	simpleEnglish withIndexDo: [:eng :ind | 
		eng = prev ifFalse: [eng]
			ifTrue: ["both say 'a month ago'"
				prevPair _ self dateAndTimeFromSeconds: 
						(arrayOfSeconds at: ind-1).
				myPair _ self dateAndTimeFromSeconds: 
						(arrayOfSeconds at: ind).
				(final at: ind-1) = prev ifTrue: ["only has 'a month ago'"
					final at: ind-1 put: 
							(final at: ind-1), ', ', prevPair first mmddyyyy].
				final at: ind put: 
							(final at: ind), ', ', myPair first mmddyyyy.
				prevPair first = myPair first 
					ifTrue: [
						(final at: ind-1) last == $m ifFalse: ["date but no time"
							final at: ind-1 put: 
								(final at: ind-1), ', ', prevPair second printMinutes].
						final at: ind put: 
							(final at: ind), ', ', myPair second printMinutes]].
		prev _ eng].
	^ final! !

!TTime class methodsFor: 'benchmarks' stamp: 'BP 3/30/2001 15:25'!
benchmarkMillisecondClock		"Time benchmarkMillisecondClock"
	"Benchmark the time spent in a call to Time>>millisecondClockValue.
	On the VM level this tests the efficiency of calls to ioMSecs()."
	"PII/400 Windows 98: 0.725 microseconds per call"
	| temp1 temp2 temp3 delayTime nLoops time |
	delayTime _ 5000. "Time to run benchmark is approx. 2*delayTime"

	"Don't run the benchmark if we have an active delay since
	we will measure the additional penalty in the primitive dispatch
	mechanism (see #benchmarkPrimitiveResponseDelay)."
	Delay anyActive ifTrue:[
		^self notify:'Some delay is currently active.
Running this benchmark will not give any useful result.'].

	"Flush the cache for this benchmark so we will have
	a clear cache hit for each send to #millisecondClockValue below"
	Object flushCache.
	temp1 _ 0.
	temp2 _ self. "e.g., temp1 == Time"
	temp3 _ self millisecondClockValue + delayTime.

	"Now check how often we can run the following loop in the given time"
	[temp2 millisecondClockValue < temp3]
		whileTrue:[temp1 _ temp1 + 1].

	nLoops _ temp1. "Remember the loops we have run during delayTime"

	"Setup the second loop"
	temp1 _ 0.
	temp3 _ nLoops.

	"Now measure how much time we spend without sending #millisecondClockValue"
	time _ Time millisecondClockValue.
	[temp1 < temp3]
		whileTrue:[temp1 _ temp1 + 1].
	time _ Time millisecondClockValue - time.

	"And compute the number of microseconds spent per call to #millisecondClockValue"
	^((delayTime - time * 1000.0 / nLoops) truncateTo: 0.001) printString,
		' microseconds per call to Time>>millisecondClockValue'! !

!TTime class methodsFor: 'benchmarks' stamp: 'BP 3/30/2001 15:25'!
benchmarkPrimitiveResponseDelay	"Time benchmarkPrimitiveResponseDelay"
	"Benchmark the overhead for primitive dispatches with an active Delay.
	On the VM level, this tests the efficiency of ioLowResMSecs."

	"PII/400 Windows98: 0.128 microseconds per prim"

	"ar 9/6/1999: This value is *extremely* important for stuff like sockets etc.
	I had a bad surprise when Michael pointed this particular problem out:
	Using the hardcoded clock() call for ioLowResMSecs on Win32 resulted in an overhead
	of 157.4 microseconds per primitive call - meaning you can't get no more than
	approx. 6000 primitives per second on my 400Mhz PII system with an active delay!!
	BTW, it finally explains why Squeak seemed soooo slow when running PWS or 
	other socket stuff. The new version (not using clock() but some Windows function) 
	looks a lot better (see above; approx. 8,000,000 prims per sec with an active delay)."

	| nLoops bb index baseTime actualTime delayTime |
	delayTime _ 5000. "Time to run this test is approx. 3*delayTime"

	Delay anyActive ifTrue:[
		^self notify:'Some delay is currently active.
Running this benchmark will not give any useful result.'].

	bb _ Array new: 1. "The object we send the prim message to"

	"Compute the # of loops we'll run in a decent amount of time"
	[(Delay forMilliseconds: delayTime) wait] 
		forkAt: Processor userInterruptPriority.

	nLoops _ 0.
	[Delay anyActive] whileTrue:[
		bb basicSize; basicSize; basicSize; basicSize; basicSize; 
			basicSize; basicSize; basicSize; basicSize; basicSize.
		nLoops _ nLoops + 1.
	].

	"Flush the cache and make sure #basicSize is in there"
	Object flushCache.
	bb basicSize.

	"Now run the loop without any active delay
	for getting an idea about its actual speed."
	baseTime _ self millisecondClockValue.
	index _ nLoops.
	[index > 0] whileTrue:[
		bb basicSize; basicSize; basicSize; basicSize; basicSize; 
			basicSize; basicSize; basicSize; basicSize; basicSize.
		index _ index - 1.
	].
	baseTime _ self millisecondClockValue - baseTime.

	"Setup the active delay but try to never make it active"
	[(Delay forMilliseconds: delayTime + delayTime) wait] 
		forkAt: Processor userInterruptPriority.

	"And run the loop"
	actualTime _ self millisecondClockValue.
	index _ nLoops.
	[index > 0] whileTrue:[
		bb basicSize; basicSize; basicSize; basicSize; basicSize; 
			basicSize; basicSize; basicSize; basicSize; basicSize.
		index _ index - 1.
	].
	actualTime _ self millisecondClockValue - actualTime.

	"And get us some result"
	^((actualTime - baseTime) * 1000 asFloat / (nLoops * 10) truncateTo: 0.001) printString,
		' microseconds overhead per primitive call'! !

!TTime class methodsFor: 'squeak protocol' stamp: 'BP 3/30/2001 15:37'!
current

	^self now! !

!TTime class methodsFor: 'squeak protocol' stamp: 'BP 3/30/2001 16:08'!
hour: hour

	^self hour: hour minute: 0 second: 0
! !

!TTime class methodsFor: 'squeak protocol' stamp: 'BP 3/30/2001 15:52'!
hour: hour minute: minute second: second

	| today time |

	today _ DateAndTime current.
	time _ Duration days: 0 hours: hour minutes: minute seconds: second.

	^self new 
		jdn: today julianDayNumber seconds: time asSeconds nanos: 0 offset: today offset;
		yourself
! !

!TTime class methodsFor: 'squeak protocol' stamp: 'BP 3/30/2001 16:42'!
seconds: seconds
	"Answer a Time from midnight"

	| d |
	d _ Duration seconds: seconds.

	^self hour: d hours minute: d minutes second: d seconds
! !

!TTime class methodsFor: 'squeak protocol' stamp: 'BP 3/30/2001 16:28'!
year: year day: dayOfYear

	^self shouldNotImplement
! !

!TTime class methodsFor: 'squeak protocol' stamp: 'BP 3/30/2001 16:29'!
year: year month: month day: day

	^self shouldNotImplement ! !

!TTime class methodsFor: 'squeak protocol' stamp: 'BP 3/30/2001 16:30'!
year: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount offset: offset

	^self shouldNotImplement! !


!Utilities class methodsFor: 'identification' stamp: 'BP 3/30/2001 16:22'!
monthDayTimeStringFrom: aSecondCount
	| aDate aTime |
	"From the date/time represented by aSecondCount, produce a string which indicates the date and time in the form:
		ddMMMhhmmPP	  where:
							dd is a two-digit day-of-month,
							MMM is the alpha month abbreviation,
							hhmm is the time,
							PP is either am or pm

          Utilities monthDayTimeStringFrom: Time primSecondsClock
"

	aDate _ Date fromSeconds: aSecondCount.
	aTime _ Time fromSeconds: aSecondCount \\ 86400.

	^ (aDate dayOfMonth asTwoCharacterString), 
		(aDate monthName copyFrom: 1 to: 3), 
		((aTime hour \\ 12) asTwoCharacterString), 
		(aTime minute asTwoCharacterString),
		(aTime hour > 12 ifTrue: ['pm'] ifFalse: ['am'])! !


!WatchMorph methodsFor: 'drawing' stamp: 'BP 3/30/2001 16:22'!
drawOn: aCanvas
	| pHour pMin pSec time |
	time _ Time now.
	pHour _ self radius: 0.6 hourAngle: time hour + (time minute/60.0).
	pMin _ self radius: 0.72 hourAngle: (time minute / 5.0).
	pSec _ self radius: 0.8 hourAngle: (time second / 5.0).
	time hour < 12
		ifTrue: [self centerColor: Color veryLightGray]
		ifFalse: [self centerColor: Color darkGray].

	antialias ifTrue: [
		aCanvas asBalloonCanvas
			aaLevel: 4;
			drawOval: (bounds insetBy: borderWidth // 2 + 1) color: self fillStyle
				borderWidth: borderWidth borderColor: borderColor;
			drawOval: (bounds insetBy: self extent*0.35) color: cColor
				borderWidth: 0 borderColor: Color black;
			drawPolygon: {self center. pHour}
				color: Color transparent borderWidth: 3 borderColor: handsColor;
			drawPolygon: {self center. pMin}
				color: Color transparent borderWidth: 2 borderColor: handsColor;
			drawPolygon: {self center. pSec}
				color: Color transparent borderWidth: 1 borderColor: handsColor]
		ifFalse: [
			super drawOn: aCanvas.
			aCanvas
				fillOval: (bounds insetBy: self extent*0.35) color: cColor;
				line: self center to: pHour width: 3 color: handsColor;
				line: self center to: pMin width: 2 color: handsColor;
				line: self center to: pSec width: 1 color: handsColor.]
! !

!ZipArchiveMember methodsFor: 'private' stamp: 'BP 3/30/2001 16:20'!
unixToDosTime: unixTime
	| dosTime dateTime secs |
	secs _ self unixToSqueakTime: unixTime.	"Squeak time (PST?)"
	dateTime _ Time dateAndTimeFromSeconds: secs.
	dosTime _ (dateTime second second) bitShift: -1.
	dosTime _ dosTime + ((dateTime second minute) bitShift: 5).
	dosTime _ dosTime + ((dateTime second hour) bitShift: 11).
	dosTime _ dosTime + ((dateTime first dayOfMonth) bitShift: 16).
	dosTime _ dosTime + ((dateTime first monthIndex) bitShift: 21).
	dosTime _ dosTime + (((dateTime first year) - 1980) bitShift: 25).
	^dosTime
! !


!TDate class reorganize!
('smalltalk-80' fromDays: fromSeconds: newDay:month:year: newDay:year:)
('squeak protocol' dateAndTimeNow fromString: julianDayNumber: readFrom: starting: starting:duration: starting:ending: today year:day: year:month:day:)
!


!TDate reorganize!
('private' gregorian)
('printing' mmddyyyy printFormat: printOn: printOn:format: storeOn:)
('smalltalk-80' addDays: subtractDate: subtractDays:)
('squeak protocol' asDate asSeconds daysLeftInYear julianDayNumber)
('ansi protocol' + - dayOfMonth dayOfWeek dayOfWeekName dayOfYear)
!

"Postscript:
Rename temporary classes TDate and TTime to Date and Time."

| oldref |

SystemOrganization removeElement: TTime name.
oldref _ Smalltalk associationAt: TTime name.
Smalltalk removeKey: TTime name.
oldref key: #Time.
Smalltalk add: oldref.
Smalltalk flushClassNameCache.
Time setName: #Time.

SystemOrganization removeElement: TDate name.
oldref _ Smalltalk associationAt: TDate name.
Smalltalk removeKey: TDate name.
oldref key: #Date.
Smalltalk add: oldref.
Smalltalk flushClassNameCache.
Date setName: #Date.

Duration initialize.
Month initialize.
Timespan initialize.

!

